//
//  StreamMonitorController.m
//  GlowWorm
//
//  Created by Curtis Jones on 2007.02.21.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "StreamMonitorController.h"

#import <Common.h>
#import <Logger.h>
#import <GlowWorm.h>
#import <GlowWormStat.h>
#import <GlowWormPlugin.h>




@implementation StreamMonitorController





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark initalization functions





- (id)initWithPcktId:(NSNumber *)aPcktId
{
	self = [super init];
	
	if (self) {
		mPcktId = [aPcktId retain];
		mSocketSrc = [[StreamMonitorSocketTableDataSource alloc] init];
		mList = [[NSMutableArray alloc] initWithCapacity:10];
		mListByName = [[NSMutableDictionary alloc] initWithCapacity:10];
		mDataView = nil;
		
		if (![NSBundle loadNibNamed:@"StreamMonitor" owner:self])  {
			NSLog(@"Failed to load StreamMonitor.nib");
			[self release];
			return nil;
		}
	}
	
	return self;
}

- (void)dealloc
{
	[mPcktId release];
	[mList release];
	[mListByName release];
	[mSocketSrc release];
	
	[super dealloc];
}

- (void)awakeFromNib
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	[mMonitorWin makeKeyAndOrderFront:nil];
	
	[mDataView retain];
	[mDataView setScrollView:mDataScrollView];
	[mSocketSrc setList:mList];
	[mSocketTbl setDataSource:mSocketSrc];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(socketSelection:) name:@"NSTableViewSelectionDidChangeNotification" object:mSocketTbl];
	
	[pool release];
}

- (void)stop
{
	mHault = TRUE;
	[mMonitorWin close];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark miscellaneous functions




- (void)addData
{
	[mDataView addText];
}

- (void)addData:(GlowWormPckt *)aPacket
{
	StreamMonitorDataView * dataView = [[mListByName objectForKey:[aPacket hrRemAddrPort]] retain];
	
	if (dataView == nil) {
		GWLog(@"%s.. adding new view for pcktId = %@", __PRETTY_FUNCTION__, [aPacket hrRemAddrPort]);
		
		dataView = [[StreamMonitorDataView alloc] initWithFrame:[mDataView frame]];
		[dataView setScrollView:mDataScrollView];
		[dataView setName:[aPacket hrRemAddrPort]];
		[mListByName setObject:dataView forKey:[aPacket hrRemAddrPort]];
		[mList addObject:dataView];
	}
	
	[dataView addPckt:aPacket];
	[dataView release];
	[mSocketTbl reloadData];
}




///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark kext miscellaneous functions




- (IBAction)actionPause:(id)sender
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
#ifdef STANDALONE
	[mDataView addText];
#endif
}

- (IBAction)actionClear:(id)sender
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	int row = [mSocketTbl selectedRow];
	StreamMonitorDataView * dataView = nil;
	
	if (row < 0 || row >= [mList count])
		return;
	
	if (nil == (dataView = [mList objectAtIndex:row]))
		return;
	
	[dataView clear];
	[mDataScrollView setNeedsDisplay:TRUE];
}

- (void)socketSelection:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	int row = [mSocketTbl selectedRow];
	StreamMonitorDataView * dataView = nil;
	
	if (row < 0 || row >= [mList count])
		dataView = mDataView;
	else
		dataView = [mList objectAtIndex:row];
	
	[mDataScrollView setDocumentView:dataView];
	[mDataScrollView setNeedsDisplay:TRUE];
	
	if (dataView == mDataView) {
		[mClearBtn setEnabled:FALSE];
	}
	else {
		[mClearBtn setEnabled:TRUE];
	}
}

@end
