//
//  StreamMonitorDataView.m
//  GlowWorm
//
//  Created by Curtis Jones on 2007.02.22.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "StreamMonitorDataView.h"
#import "StreamMonitorPacket.h"
#import <Common.h>
#import <Logger.h>
#import <GlowWorm.h>


@implementation StreamMonitorDataView

- (id)initWithFrame:(NSRect)frameRect
{
	GWLog(@"%s.. x=%d, y=%d, h=%d, w=%d", __PRETTY_FUNCTION__, (int)frameRect.origin.x, (int)frameRect.origin.y, (int)frameRect.size.height, (int)frameRect.size.width);
	
	self = [super initWithFrame:frameRect];
	
	base		= 0;
	count		= 0;
	bcount	= 0;
	frameh	= frameRect.size.height;
	scrollh = 0;
	
	mName = nil;
	mScrollView = nil;
	
	[self setPostsFrameChangedNotifications:TRUE];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewFrameDidChange:) name:@"NSViewFrameDidChangeNotification" object:self];
	
	return self;
}

- (void)dealloc
{
	[mScrollView release];
	[mName release];
	
	[super dealloc];
}

- (void)setScrollView:(NSScrollView *)aScrollView
{
	if (mScrollView != nil)
		[mScrollView release];
	
	mScrollView = [aScrollView retain];
	scrollh = [mScrollView frame].size.height;
}

- (void)setName:(NSString *)aName
{
	if (mName != nil)
		[mName release];
	
	mName = [aName retain];
}

- (NSString *)name
{
	return mName;
}

- (uint32_t)xfer
{
	return bcount;
}

- (void)clear
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	uint32_t i=0;
	NSArray * subviews = [NSArray arrayWithArray:[self subviews]];
	
	for (i = 0; i < [subviews count]; ++i)
		[[subviews objectAtIndex:i] removeFromSuperviewWithoutNeedingDisplay];
	
	NSRect frame = [self frame];
	frame.size.height = 0;
	frameh = 0;
	[self setFrameSize:frame.size];
}

- (void)addText
{
	u_char value = 0;
	int i=0, leng = (random() % 100) + 1;
	NSMutableData * data = [NSMutableData dataWithCapacity:leng];
	
	for (; i < leng; ++i) {
		value = random() % 0xFF;
		[data appendBytes:&value length:1];
	}
	
	GlowWormPckt * pckt = [[GlowWormPckt alloc] init];
	struct timeval tv;
	uint32_t direction = (random() % 2) ? GWS_CONN_DATA_INCOMING : GWS_CONN_DATA_OUTGOING;
	
	gettimeofday(&tv, NULL);
	
	[pckt setObject:[NSNumber numberWithUnsignedLong:0x12345678] forKey:@"loc_addr"];
	[pckt setObject:[NSNumber numberWithUnsignedLong:1234] forKey:@"loc_port"];
	[pckt setObject:[NSNumber numberWithUnsignedLong:0x78563412] forKey:@"rem_addr"];
	[pckt setObject:[NSNumber numberWithUnsignedLong:4321] forKey:@"rem_port"];
	[pckt setObject:[NSNumber numberWithUnsignedLong:direction] forKey:@"direction"];
	[pckt setObject:[NSNumber numberWithUnsignedLongLong:((tv.tv_sec*1000000)+tv.tv_usec)] forKey:@"time"];
	[pckt setObject:data forKey:@"data"];
	
	[self addPckt:pckt];
	
	[pckt release];
}

- (void)addText:(NSData *)data
{
	if (data == nil || 0 == [data length])
		return;
	
	StreamMonitorPacket * smp = [[StreamMonitorPacket alloc] initWithOffset:bcount data:data];
	[smp setupViews:(base==0?base:base+5) view:self];
	
	base += (base==0?0:5) + [smp height];
	bcount += [data length];
	frameh = [self frame].size.height;
	
//[self addText:data];
}

- (void)addPckt:(GlowWormPckt *)aPckt
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	if (aPckt == nil || [aPckt data] == nil || [[aPckt data] length] == 0)
		return;
	
	StreamMonitorPacket * smp = [[StreamMonitorPacket alloc] initWithOffset:bcount andPckt:aPckt];
	
	[smp setupViews:(base==0?base:base+5) view:self];
	
	base   += (base==0?0:5) + [smp height];
	bcount += [[aPckt data] length];
	frameh  = [self frame].size.height;
}

- (void)viewFrameDidChange:(NSNotification *)aNotification
{
	/*
	NSRect frame = [self frame];
	
	GWLog(@"%s o.x=%d, o.y=%d, s.w=%d, s.h=%d | s.h=%d, frameh=%d, base=%lu", __PRETTY_FUNCTION__,
				(int)frame.origin.x, (int)frame.origin.y,
				(int)frame.size.width, (int)frame.size.height,
				(int)[mScrollView frame].size.height,
				(int)frameh, (uint32_t)base);
	*/
	
	/*
	if ([mScrollView frame].size.height != scrollh) {
		if ([mScrollView frame].size.height < base) {
			GWLog(@"%s.. setting %d back to %d", __PRETTY_FUNCTION__, (int)frame.size.height, (int)frameh);
			frame.size.height = (float)frameh;
			[self setFrame:frame];
		}
		else {
			base = 0;
			
			int i=0, difheight = frame.size.height - frameh;
			NSArray * subviews = [self subviews];
			NSRect aframe;
			
			GWLog(@"%s.. difheight = %d", __PRETTY_FUNCTION__, difheight);
			
			for (i = 0; i < [subviews count]; ++i) {
				NSView * nsview = [subviews objectAtIndex:i];
				
				aframe = [nsview frame];
				aframe.origin.y += difheight;
				
				[nsview setFrameOrigin:aframe.origin];
				
				if (frame.size.height - aframe.origin.y > base)
					base = frame.size.height - aframe.origin.y;
			}
			
			// visibleRect
			// scrollPoint:
			
			NSRect vrect = [self visibleRect];
			vrect.origin.y += difheight;
			[self scrollPoint:vrect.origin];
			
			frameh = frame.size.height;
		}
		
		scrollh = [mScrollView frame].size.height;
	}
	*/
	
	/*
	// if the scroll height > current base
	//   move data to the top
	// else
	//   put the frame height back the way it was
	//
//if ([mScrollView frame].size.height < frameh) {
	if ([mScrollView frame].size.height < base) {
		GWLog(@"%s.. setting %d back to %d", __PRETTY_FUNCTION__, (int)frame.size.height, (int)frameh);
		frame.size.height = (float)frameh;
//	[self setBoundsSize:frame.size];
		[self setFrame:frame];
	}
//if ([mScrollView frame].size.height > frameh) {
//else if (frame.size.height > frameh) {
//	if (frame.size.height > frameh) {
			base = 0;
			
			int i=0, difheight = frame.size.height - frameh;
			NSArray * subviews = [self subviews];
			NSRect aframe;
			
			GWLog(@"%s.. difheight = %d", __PRETTY_FUNCTION__, difheight);
			
			for (i = 0; i < [subviews count]; ++i) {
				NSView * nsview = [subviews objectAtIndex:i];
				
				aframe = [nsview frame];
				aframe.origin.y += difheight;
				
				[nsview setFrameOrigin:aframe.origin];
				
				if (frame.size.height - aframe.origin.y > base)
					base = frame.size.height - aframe.origin.y;
			}
			
			frameh = frame.size.height;
//	}
	}
//else {
//	frame.size.height = (float)frameh;
//	[self setBoundsSize:frame.size];
//}
	*/
}

@end
