//
//  StreamMonitorPluginController.m
//  GlowWorm
//
//  Created by Curtis Jones on 2007.02.21.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "StreamMonitorPluginController.h"

#import <Common.h>
#import <Logger.h>
#import <GlowWorm.h>
#import <StatTrans.h>
#import <GlowWormStat.h>
#import <GlowWormPlugin.h>





@implementation StreamMonitorPluginController





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark initalization functions





- (id)init
{
	self = [super init];
	
	if (self) {
		mMonitorDict = [[NSMutableDictionary dictionaryWithCapacity:10] retain];
	}
	
	return self;
}

- (void)dealloc
{
	[super dealloc];
}

- (void)awakeFromNib
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	// load our sub-menu
	{
		mMenuMainMenu  = [[[NSMenu alloc] init] autorelease];
		mMenuMainItem  = [[[NSMenuItem alloc] init] autorelease];
		mMenuPrefsItem = [[[NSMenuItem alloc] init] autorelease];
	//mMenuSetupItem = [[[NSMenuItem alloc] init] autorelease];
		
		[mMenuMainItem setTitle:@"Stream Monitor"];
		[mMenuMainItem setEnabled:TRUE];
		
		[mMenuPrefsItem setTarget:self];
		[mMenuPrefsItem setAction:@selector(actionPreferences:)];
		[mMenuPrefsItem setTitle:@"Preferences..."];
		[mMenuPrefsItem setEnabled:TRUE];
		
		/*
		[mMenuSetupItem setTarget:self];
		[mMenuSetupItem setAction:@selector(actionMonitor:)];
		[mMenuSetupItem setTitle:@"Monitor Window"];
		[mMenuSetupItem setEnabled:TRUE];
		*/
		
		[[GlowWormPlugin pluginsMenu] addItem:mMenuMainItem];
		[[GlowWormPlugin pluginsMenu] setSubmenu:mMenuMainMenu forItem:mMenuMainItem];
		
		[mMenuMainMenu addItem:mMenuPrefsItem];
	//[mMenuMainMenu addItem:mMenuRulesItem];
	}
	
#ifndef STANDALONE
	[self startup];
#else
	StreamMonitorController * smc = [[StreamMonitorController alloc] initWithPcktId:[NSNumber numberWithInt:1]];
#endif
	
	[pool release];
}

- (void)startup
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	//
	// register to be notified if any preference values 
	// change, and set the default values for those 
	// preferences if there is no current value.
	//
	{
		[[NSNotificationCenter defaultCenter] addObserver:self
																						 selector:@selector(actionPluginDisableCallback:)
																								 name:@"actionPluginDisable"
																							 object:nil];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
																						 selector:@selector(actionPluginEnableCallback:)
																								 name:@"actionPluginEnable"
																							 object:nil];
		
		if (nil == [[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"showHostNamesTrans"])
			[[[NSUserDefaultsController sharedUserDefaultsController] values] setValue:[NSNumber numberWithBool:TRUE] forKey:@"showHostNamesTrans"];
		
		if (nil == [[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"showServiceNamesTrans"])
			[[[NSUserDefaultsController sharedUserDefaultsController] values] setValue:[NSNumber numberWithBool:TRUE] forKey:@"showServiceNamesTrans"];
		
		if (nil == [[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"socketBufferSizeTrans"])
			[[[NSUserDefaultsController sharedUserDefaultsController] values] setValue:[NSNumber numberWithInt:10] forKey:@"socketBufferSizeTrans"];
		
		mPrefShowHostNames		= [[[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"showHostNamesTrans"		] boolValue];
		mPrefShowServiceNames = [[[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"showServiceNamesTrans"	] boolValue];
		mPrefSocketBufferSize =  [[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:@"socketBufferSizeTrans"	];
		
		//[mRuleSrc showServiceNames:mPrefShowServiceNames];
	}
	
	[self startupLoadRules];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	GWLog(@"%s.. keyPath=%@", __PRETTY_FUNCTION__, keyPath);
	
	// showHostNamesTrans
	//
	if ([keyPath isEqualToString:@"values.showHostNamesTrans"]) {
		mPrefShowHostNames = [[[object values] valueForKey:@"showHostNamesTrans"] boolValue];
//	[mRuleSrc showServiceNames:mPrefShowHostNames];
//	[mRuleTbl reloadData];
	}
	
	// showServiceNamesTrans
	//
	else if ([keyPath isEqualToString:@"values.showServiceNamesTrans"]) {
		mPrefShowServiceNames = [[[object values] valueForKey:@"showServiceNamesTrans"] boolValue];
	}
	
	// socketBufferSizeTrans
	//
	else if ([keyPath isEqualToString:@"values.socketBufferSizeTrans"]) {
		mPrefSocketBufferSize = [[object values] valueForKey:@"socketBufferSizeTrans"];
	}
	
	// unsupported
	//
	else
		GWLog(@"%s.. unsupported keyPath, %@", __PRETTY_FUNCTION__, keyPath);
}

- (void)startupLoadRules
{
	[GlowWormPckt pcktInsertHandler:self callbackSel:@selector(pcktCallback:) callbackCtx:nil];
	[GlowWormRule insertRuleChangeHandlerForList:0 andTerm:GWR_TERM_PERM callbackObj:self callbackSel:@selector(ruleChangeCallback:)];
	[GlowWormRule selectRule:0 list:GWR_LIST_CONN term:GWR_TERM_PERM callbackObj:self callbackSel:@selector(ruleChangeCallback:)];
}

- (StreamMonitorController *)newDataMonitorForRuleId:(NSNumber *)aRuleId
{
	if (aRuleId == nil)
		return nil;
	
	StreamMonitorController * smc = [[StreamMonitorController alloc] initWithPcktId:aRuleId];
	
	[mMonitorDict setObject:smc forKey:aRuleId];
	
	return [smc autorelease];
}




///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark action functions




- (IBAction)actionPreferences:(id)sender
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	//[mPrefWin makeKeyAndOrderFront:sender];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark observer callback functions





- (void)actionPluginDisableCallback:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	//[mMainWin saveFrameUsingName:@"GlowWorm Plugin (Data Monitor)"];
	//[mDataSrc delAll];
}

- (void)actionPluginEnableCallback:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	[self startupLoadRules];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark kext callback functions




- (void)pcktCallback:(StatTrans *)aTrans
{
	//GWLog(@"%s", __PRETTY_FUNCTION__);
	
	StreamMonitorController * smc = nil;
	GlowWormPckt * pckt = (GlowWormPckt *)[aTrans body];
//GlowWormMesg * mesg = (GlowWormMesg *)[aTrans mesg];
	
	if (pckt == nil) {
		//GWLog(@"%s.. handler confirmation callback; act3=0x%08lx", __PRETTY_FUNCTION__, [mesg act3]);
		return;
	}
	
	@synchronized (mMonitorDict)
	{
		smc = [[mMonitorDict objectForKey:[pckt pcktId]] retain];
		
		if (smc == nil)
			smc = [[self newDataMonitorForRuleId:[pckt pcktId]] retain];
	}
	
	[smc addData:pckt];
	[smc release];
}

- (void)ruleChangeCallback:(RuleTrans *)aTrans
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	GlowWormRule * rule = (GlowWormRule *)[aTrans body];
//GlowWormMesg * mesg = (GlowWormMesg *)[aTrans mesg];
	
	if (rule == nil)
		return;
	
	NSNumber * pcktId = [rule uniqueId];
	NSNumber * pckts = [rule pckts];
	
	@synchronized (mMonitorDict)
	{
		StreamMonitorController * smc = [mMonitorDict objectForKey:pcktId];
		
		if (smc != nil && (GWR_PCKTS_OFF == [pckts intValue] || 0 == [pckts intValue])) {
			[smc stop];
			[mMonitorDict removeObjectForKey:pcktId];
		}
		else if (smc == nil && GWR_PCKTS_ON == [pckts intValue])
			smc = [self newDataMonitorForRuleId:pcktId];
	}
}

@end
